@@ -3,7 +3,7 @@
 
 /*
 
-  Copyright 2012-2015 Jochen Topf <jochen@topf.org>.
+  Copyright 2012-2016 Jochen Topf <jochen@topf.org>.
 
   This file is part of OSMCoastline.
 
@@ -34,7 +34,7 @@ class OGRMultiPolygon;
 class OGREnvelope;
 class OutputDatabase;
 
-typedef std::vector<OGRPolygon*> polygon_vector_type;
+typedef std::vector<std::unique_ptr<OGRPolygon>> polygon_vector_type;
 
 /**
  * A collection of land polygons created out of coastlines.
@@ -74,12 +74,13 @@ class CoastlinePolygons {
 
     std::unique_ptr<OGRPolygon> create_rectangular_polygon(double x1, double y1, double x2, double y2, double expand=0) const;
 
-    void split_geometry(std::unique_ptr<OGRGeometry> geom, int level);
-    void split_polygon(OGRPolygon* polygon, int level);
+    void split_geometry(std::unique_ptr<OGRGeometry>&& geom, int level);
+    void split_polygon(std::unique_ptr<OGRPolygon>&& polygon, int level);
     void split_bbox(OGREnvelope e, polygon_vector_type&& v);
 
-    bool add_segment_to_line(OGRLineString* line, OGRPoint* point1, OGRPoint* point2);
-    void output_polygon_ring_as_lines(int max_points, OGRLinearRing* ring);
+    void add_line_to_output(std::unique_ptr<OGRLineString> line, OGRSpatialReference* srs) const;
+    bool add_segment_to_line(OGRLineString* line, OGRPoint* point1, OGRPoint* point2) const;
+    void output_polygon_ring_as_lines(int max_points, const OGRLinearRing* ring) const;
 
 public:
 
@@ -120,7 +121,7 @@ public:
     unsigned int output_water_polygons();
 
     /// Write all coastlines to the output database (as lines).
-    void output_lines(int max_points);
+    void output_lines(int max_points) const;
 
 };
 
