@@ -3,7 +3,7 @@
 
 /*
 
-  Copyright 2012-2015 Jochen Topf <jochen@topf.org>.
+  Copyright 2012-2016 Jochen Topf <jochen@topf.org>.
 
   This file is part of OSMCoastline.
 
@@ -105,7 +105,7 @@ public:
 
     unsigned int check_positions(bool output_missing);
 
-    void add_polygons_to_vector(std::vector<OGRGeometry*>& vector);
+    std::vector<OGRGeometry*> add_polygons_to_vector();
 
     unsigned int output_rings(OutputDatabase& output);
 
