@@ -1,6 +1,6 @@
 /*
 
-  Copyright 2012-2015 Jochen Topf <jochen@topf.org>.
+  Copyright 2012-2016 Jochen Topf <jochen@topf.org>.
 
   This file is part of OSMCoastline.
 
@@ -25,7 +25,7 @@
 #include <cstdlib>
 #include <getopt.h>
 
-#include "osmcoastline.hpp"
+#include "return_codes.hpp"
 #include "options.hpp"
 
 Options::Options(int argc, char* argv[]) :
@@ -62,11 +62,12 @@ Options::Options(int argc, char* argv[]) :
         {"srs",             required_argument, 0, 's'},
         {"write-segments",  required_argument, 0, 'S'},
         {"verbose",               no_argument, 0, 'v'},
+        {"version",               no_argument, 0, 'V'},
         {0, 0, 0, 0}
     };
 
     while (1) {
-        int c = getopt_long(argc, argv, "b:c:idhlm:o:p:rfs:S:v", long_options, 0);
+        int c = getopt_long(argc, argv, "b:c:idhlm:o:p:rfs:S:vV", long_options, 0);
         if (c == -1)
             break;
 
@@ -131,6 +132,13 @@ Options::Options(int argc, char* argv[]) :
             case 'v':
                 verbose = true;
                 break;
+            case 'V':
+                std::cout << "osmcoastline version " OSMCOASTLINE_VERSION "\n"
+                          << "Copyright (C) 2012-2016  Jochen Topf <jochen@topf.org>\n"
+                          << "License: GNU GENERAL PUBLIC LICENSE Version 3 <http://gnu.org/licenses/gpl.html>.\n"
+                          << "This is free software: you are free to change and redistribute it.\n"
+                          << "There is NO WARRANTY, to the extent permitted by law.\n";
+                exit(return_code_ok);
             default:
                 exit(return_code_cmdline);
         }
@@ -197,6 +205,7 @@ void Options::print_help() const {
               << "  -s, --srs=EPSGCODE         - Set SRS (4326 for WGS84 (default) or 3857)\n"
               << "  -S, --write-segments=FILE  - Write segments to given file\n"
               << "  -v, --verbose              - Verbose output\n"
+              << "  -V, --version              - Show version and exit\n"
               << "\n";
 }
 
