@@ -1,6 +1,6 @@
 /*
 
-  Copyright 2012-2015 Jochen Topf <jochen@topf.org>.
+  Copyright 2012-2016 Jochen Topf <jochen@topf.org>.
 
   This file is part of OSMCoastline.
 
@@ -19,22 +19,26 @@
 
 */
 
+#include <algorithm>
 #include <iostream>
 #include <set>
 #include <getopt.h>
 
 #include <osmium/io/any_input.hpp>
+#include <osmium/io/input_iterator.hpp>
+#include <osmium/io/output_iterator.hpp>
 #include <osmium/io/pbf_output.hpp>
 #include <osmium/handler.hpp>
 #include <osmium/osm/entity_bits.hpp>
 
-#include "osmcoastline.hpp"
+#include "return_codes.hpp"
 
 void print_help() {
     std::cout << "osmcoastline_filter [OPTIONS] OSMFILE\n"
               << "\nOptions:\n"
               << "  -h, --help           - This help message\n"
               << "  -o, --output=OSMFILE - Where to write output (default: none)\n"
+              << "  -V, --version        - Show version and exit\n"
               << "\n";
 }
 
@@ -44,11 +48,12 @@ int main(int argc, char* argv[]) {
     static struct option long_options[] = {
         {"help",         no_argument, 0, 'h'},
         {"output", required_argument, 0, 'o'},
+        {"version",      no_argument, 0, 'V'},
         {0, 0, 0, 0}
     };
 
     while (1) {
-        int c = getopt_long(argc, argv, "ho:", long_options, 0);
+        int c = getopt_long(argc, argv, "ho:V", long_options, 0);
         if (c == -1)
             break;
 
@@ -59,6 +64,13 @@ int main(int argc, char* argv[]) {
             case 'o':
                 output_filename = optarg;
                 break;
+            case 'V':
+                std::cout << "osmcoastline_filter version " OSMCOASTLINE_VERSION "\n"
+                          << "Copyright (C) 2012-2016  Jochen Topf <jochen@topf.org>\n"
+                          << "License: GNU GENERAL PUBLIC LICENSE Version 3 <http://gnu.org/licenses/gpl.html>.\n"
+                          << "This is free software: you are free to change and redistribute it.\n"
+                          << "There is NO WARRANTY, to the extent permitted by law.\n";
+                exit(return_code_ok);
             default:
                 exit(return_code_fatal);
         }
@@ -82,55 +94,39 @@ int main(int argc, char* argv[]) {
 
     try {
         osmium::io::Writer writer(output_filename, header);
+        auto output_it = osmium::io::make_output_iterator(writer);
 
-        std::set<osmium::object_id_type> ids;
-        osmium::memory::Buffer output_buffer(10240);
+        std::vector<osmium::object_id_type> ids;
 
         {
             osmium::io::Reader reader(infile, osmium::osm_entity_bits::way);
-            while (auto input_buffer = reader.read()) {
-                for (auto it = input_buffer.begin<const osmium::Way>(); it != input_buffer.end<const osmium::Way>(); ++it) {
-                    const char* natural = it->get_value_by_key("natural");
-                    if (natural && !strcmp(natural, "coastline")) {
-                        output_buffer.add_item(*it);
-                        output_buffer.commit();
-                        if (output_buffer.committed() >= 10240) {
-                            osmium::memory::Buffer new_buffer(10240);
-                            std::swap(output_buffer, new_buffer);
-                            writer(std::move(new_buffer));
-                        }
-                        for (const auto& nr : it->nodes()) {
-                            ids.insert(nr.ref());
-                        }
+            auto ways = osmium::io::make_input_iterator_range<const osmium::Way>(reader);
+            for (const osmium::Way& way : ways) {
+                const char* natural = way.get_value_by_key("natural");
+                if (natural && !strcmp(natural, "coastline")) {
+                    *output_it++ = way;
+                    for (const auto& nr : way.nodes()) {
+                        ids.push_back(nr.ref());
                     }
                 }
             }
             reader.close();
         }
 
+        std::sort(ids.begin(), ids.end());
+        auto last = std::unique(ids.begin(), ids.end());
+
         {
             osmium::io::Reader reader(infile, osmium::osm_entity_bits::node);
-            while (auto input_buffer = reader.read()) {
-                for (auto it = input_buffer.begin<const osmium::Node>(); it != input_buffer.end<const osmium::Node>(); ++it) {
-                    const char* natural = it->get_value_by_key("natural");
-                    if ((ids.find(it->id()) != ids.end()) || (natural && !strcmp(natural, "coastline"))) {
-                        output_buffer.add_item(*it);
-                        output_buffer.commit();
-                        if (output_buffer.committed() >= 10240) {
-                            osmium::memory::Buffer new_buffer(10240);
-                            std::swap(output_buffer, new_buffer);
-                            writer(std::move(new_buffer));
-                        }
-                    }
-                }
-            }
+            auto nodes = osmium::io::make_input_iterator_range<const osmium::Node>(reader);
+            std::copy_if(nodes.cbegin(), nodes.cend(), output_it, [&ids, &last](const osmium::Node& node){
+                const char* natural = node.get_value_by_key("natural");
+                return std::binary_search(ids.begin(), last, node.id()) ||
+                       (natural && !strcmp(natural, "coastline"));
+            });
             reader.close();
         }
 
-        if (output_buffer.committed() > 0) {
-            writer(std::move(output_buffer));
-        }
-
         writer.close();
     } catch (osmium::io_error& e) {
         std::cerr << "io error: " << e.what() << "'\n";
